pub struct Buffer<'buf, R: Read> {
pub buf: &'buf mut [u8],
pub n_bytes: usize,
pub n_shifted_out: usize,
pub pos_shifted: LinePosition,
/* private fields */
}Expand description
A buffer for reading JSON data. Is a private struct, the “pub” is only for testing.
Fields
buf: &'buf mut [u8]The working buffer for reading JSON data.
n_bytes: usizeNumber of valid bytes in the buffer. Contract: n_bytes <= buf.len()
n_shifted_out: usizeNumber of bytes that have been shifted out of the buffer.
pos_shifted: LinePositionLine position correction due to shifting operations.
Implementations
Sourceimpl<'buf, R: Read> Buffer<'buf, R>
impl<'buf, R: Read> Buffer<'buf, R>
Sourcepub fn new(reader: &'buf mut R, buf: &'buf mut [u8]) -> Self
pub fn new(reader: &'buf mut R, buf: &'buf mut [u8]) -> Self
Creates a new buffer with the given reader and buffer.
Sourcepub fn read_more(&mut self) -> RJiterResult<usize>
pub fn read_more(&mut self) -> RJiterResult<usize>
Read from the underlying reader into the buffer.
Returns the number of bytes read.
Errors
From the underlying reader.
Sourcepub fn shift_buffer(&mut self, to_pos: usize, from_pos: usize)
pub fn shift_buffer(&mut self, to_pos: usize, from_pos: usize)
Shift the buffer to the left, and update the index and line-column position.
Arguments
to_pos: The position to shift to. Usually is 0 or is 1 for strings.from_pos: The position to shift from. The case of outside the buffer is handled.
Sourcepub fn skip_spaces(&mut self, pos: usize) -> RJiterResult<()>
pub fn skip_spaces(&mut self, pos: usize) -> RJiterResult<()>
Sourcepub fn collect_while<F>(
&mut self,
predicate: F,
start_pos: usize,
allow_shift: bool,
) -> RJiterResult<(usize, usize)>
pub fn collect_while<F>( &mut self, predicate: F, start_pos: usize, allow_shift: bool, ) -> RJiterResult<(usize, usize)>
Collect bytes while a predicate is true, starting at the given position.
Returns a tuple of (start_position, end_position) where end_position is the offset
of the first rejected byte, or EOF.
If buffer is full with all accepted bytes, it’s an error.
The function can optionally shift the buffer once to discard bytes before start_pos.
Arguments
predicate- A function that returns true if the byte should be acceptedstart_pos- The position in the buffer to start collecting fromallow_shift- If true, allows shifting the buffer once when it fills up (discards bytes beforestart_pos)
Errors
Returns ErrorType::BufferFull if the buffer fills up with all accepted bytes.
Also returns errors from the underlying reader.
Sourcepub fn collect_count(
&mut self,
count: usize,
start_pos: usize,
allow_shift: bool,
) -> RJiterResult<(usize, usize)>
pub fn collect_count( &mut self, count: usize, start_pos: usize, allow_shift: bool, ) -> RJiterResult<(usize, usize)>
Collect exactly count bytes starting at the given position, or until EOF.
Returns a tuple of (start_position, end_position) where end_position is the offset
after the collected bytes (start_pos + actual_collected).
If buffer is too small to hold the requested bytes, it’s an error.
The function can optionally shift the buffer once to discard bytes before start_pos.
Arguments
count- The number of bytes to collectstart_pos- The position in the buffer to start collecting fromallow_shift- If true, allows shifting the buffer once when it fills up (discards bytes beforestart_pos)
Errors
Returns ErrorType::BufferFull if the buffer is too small to hold the requested bytes.
Also returns errors from the underlying reader.
Sourcepub fn skip_n(
&mut self,
count: usize,
start_pos: usize,
) -> RJiterResult<(usize, usize)>
pub fn skip_n( &mut self, count: usize, start_pos: usize, ) -> RJiterResult<(usize, usize)>
Skip exactly count bytes starting at the given position, or until EOF.
Returns the new position in the buffer after skipping.
This function works incrementally and can skip any number of bytes regardless of buffer size. It repeatedly shifts and reads as needed when the buffer is too small. When bytes fit in the buffer, it just returns the new position without shifting.
Arguments
count- The number of bytes to skipstart_pos- The position in the buffer to start skipping from
Errors
Returns errors from the underlying reader.